#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG that displays interactions with Google Cloud Build.
"""

from __future__ import annotations

import os
from datetime import datetime
from pathlib import Path
from typing import Any, cast

import yaml

from airflow.models.dag import DAG
from airflow.models.xcom_arg import XComArg
from airflow.providers.google.cloud.operators.cloud_build import (
    CloudBuildCancelBuildOperator,
    CloudBuildCreateBuildOperator,
    CloudBuildGetBuildOperator,
    CloudBuildListBuildsOperator,
    CloudBuildRetryBuildOperator,
)
from airflow.providers.standard.operators.bash import BashOperator

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS

if AIRFLOW_V_3_0_PLUS:
    from airflow.sdk import task_group
else:
    # Airflow 2 path
    from airflow.decorators import task_group  # type: ignore[attr-defined,no-redef]

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID

DAG_ID = "gcp_cloud_build"

GCP_SOURCE_ARCHIVE_URL = "gs://airflow-system-tests-resources/cloud-build/file.tar.gz"
# Repository with this name is expected created within the project $SYSTEM_TESTS_GCP_PROJECT
# If you'd like to run this system test locally, please
#   1. Create Cloud Source Repository
#   2. Push into a master branch the following file:
#   providers/google/tests/system/google/cloud/cloud_build/resources/example_cloud_build.yaml
GCP_SOURCE_REPOSITORY_NAME = "test-cloud-build-repository"

CURRENT_FOLDER = Path(__file__).parent

# [START howto_operator_gcp_create_build_from_storage_body]
CREATE_BUILD_FROM_STORAGE_BODY = {
    "source": {"storage_source": GCP_SOURCE_ARCHIVE_URL},
    "steps": [{"name": "ubuntu", "entrypoint": "bash", "args": ["-c", "echo Hello world && sleep 200"]}],
}
# [END howto_operator_gcp_create_build_from_storage_body]

# [START howto_operator_create_build_from_repo_body]
CREATE_BUILD_FROM_REPO_BODY: dict[str, Any] = {
    "source": {"repo_source": {"repo_name": GCP_SOURCE_REPOSITORY_NAME, "branch_name": "master"}},
    "steps": [{"name": "ubuntu", "entrypoint": "bash", "args": ["-c", "echo Hello world && sleep 200"]}],
}
# [END howto_operator_create_build_from_repo_body]

CREATE_BUILD_FROM_REPO_NO_WAIT_BODY: dict[str, Any] = {
    "source": {"repo_source": {"repo_name": GCP_SOURCE_REPOSITORY_NAME, "branch_name": "master"}},
    "steps": [{"name": "ubuntu", "entrypoint": "bash", "args": ["-c", "echo Hello world && sleep 300"]}],
}

with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "cloud_build"],
) as dag:

    @task_group(group_id="build_from_storage")
    def build_from_storage():
        # [START howto_operator_create_build_from_storage]
        create_build_from_storage = CloudBuildCreateBuildOperator(
            task_id="create_build_from_storage",
            project_id=PROJECT_ID,
            build=CREATE_BUILD_FROM_STORAGE_BODY,
        )
        # [END howto_operator_create_build_from_storage]

        # [START howto_operator_create_build_from_storage_result]
        create_build_from_storage_result = BashOperator(
            bash_command=f"echo {cast('str', XComArg(create_build_from_storage, key='results'))}",
            task_id="create_build_from_storage_result",
        )
        # [END howto_operator_create_build_from_storage_result]

        create_build_from_storage >> create_build_from_storage_result

    @task_group(group_id="build_from_storage_deferrable")
    def build_from_storage_deferrable():
        # [START howto_operator_create_build_from_storage_async]
        create_build_from_storage = CloudBuildCreateBuildOperator(
            task_id="create_build_from_storage",
            project_id=PROJECT_ID,
            build=CREATE_BUILD_FROM_STORAGE_BODY,
            deferrable=True,
        )
        # [END howto_operator_create_build_from_storage_async]

        create_build_from_storage_result = BashOperator(
            bash_command=f"echo {cast('str', XComArg(create_build_from_storage, key='results'))}",
            task_id="create_build_from_storage_result",
        )

        create_build_from_storage >> create_build_from_storage_result

    @task_group(group_id="build_from_repo")
    def build_from_repo():
        # [START howto_operator_create_build_from_repo]
        create_build_from_repo = CloudBuildCreateBuildOperator(
            task_id="create_build_from_repo",
            project_id=PROJECT_ID,
            build=CREATE_BUILD_FROM_REPO_BODY,
        )
        # [END howto_operator_create_build_from_repo]

        # [START howto_operator_create_build_from_repo_result]
        create_build_from_repo_result = BashOperator(
            bash_command=f"echo {cast('str', XComArg(create_build_from_repo, key='results'))}",
            task_id="create_build_from_repo_result",
        )
        # [END howto_operator_create_build_from_repo_result]

        create_build_from_repo >> create_build_from_repo_result

    @task_group(group_id="build_from_repo_deferrable")
    def build_from_repo_deferrable():
        # [START howto_operator_create_build_from_repo_async]
        create_build_from_repo = CloudBuildCreateBuildOperator(
            task_id="create_build_from_repo",
            project_id=PROJECT_ID,
            build=CREATE_BUILD_FROM_REPO_BODY,
            deferrable=True,
        )
        # [END howto_operator_create_build_from_repo_async]

        create_build_from_repo_result = BashOperator(
            bash_command=f"echo {cast('str', XComArg(create_build_from_repo, key='results'))}",
            task_id="create_build_from_repo_result",
        )

        create_build_from_repo >> create_build_from_repo_result

    # [START howto_operator_gcp_create_build_from_yaml_body]
    create_build_from_file = CloudBuildCreateBuildOperator(
        task_id="create_build_from_file",
        project_id=PROJECT_ID,
        build=yaml.safe_load((Path(CURRENT_FOLDER) / "resources" / "example_cloud_build.yaml").read_text()),
        params={"name": "Airflow"},
    )
    # [END howto_operator_gcp_create_build_from_yaml_body]

    # [START howto_operator_gcp_create_build_from_yaml_body_async]
    create_build_from_file_deferrable = CloudBuildCreateBuildOperator(
        task_id="create_build_from_file_deferrable",
        project_id=PROJECT_ID,
        build=yaml.safe_load((Path(CURRENT_FOLDER) / "resources" / "example_cloud_build.yaml").read_text()),
        params={"name": "Airflow"},
        deferrable=True,
    )
    # [END howto_operator_gcp_create_build_from_yaml_body_async]

    # [START howto_operator_list_builds]
    list_builds = CloudBuildListBuildsOperator(
        task_id="list_builds",
        project_id=PROJECT_ID,
        location="global",
    )
    # [END howto_operator_list_builds]

    @task_group(group_id="no_wait_cancel_retry_get")
    def no_wait_cancel_retry_get():
        # [START howto_operator_create_build_without_wait]
        create_build_without_wait = CloudBuildCreateBuildOperator(
            task_id="create_build_without_wait",
            project_id=PROJECT_ID,
            build=CREATE_BUILD_FROM_REPO_NO_WAIT_BODY,
            wait=False,
        )
        # [END howto_operator_create_build_without_wait]

        # [START howto_operator_cancel_build]
        cancel_build = CloudBuildCancelBuildOperator(
            task_id="cancel_build",
            id_=cast("str", XComArg(create_build_without_wait, key="id")),
            project_id=PROJECT_ID,
        )
        # [END howto_operator_cancel_build]

        # [START howto_operator_retry_build]
        retry_build = CloudBuildRetryBuildOperator(
            task_id="retry_build",
            id_=cast("str", XComArg(cancel_build, key="id")),
            project_id=PROJECT_ID,
        )
        # [END howto_operator_retry_build]

        # [START howto_operator_get_build]
        get_build = CloudBuildGetBuildOperator(
            task_id="get_build",
            id_=cast("str", XComArg(retry_build, key="id")),
            project_id=PROJECT_ID,
        )
        # [END howto_operator_get_build]

        create_build_without_wait >> cancel_build >> retry_build >> get_build

    @task_group(group_id="no_wait_cancel_retry_get_deferrable")
    def no_wait_cancel_retry_get_deferrable():
        # [START howto_operator_create_build_without_wait_async]
        create_build_without_wait = CloudBuildCreateBuildOperator(
            task_id="create_build_without_wait",
            project_id=PROJECT_ID,
            build=CREATE_BUILD_FROM_REPO_NO_WAIT_BODY,
            wait=False,
            deferrable=True,
        )
        # [END howto_operator_create_build_without_wait_async]

        cancel_build = CloudBuildCancelBuildOperator(
            task_id="cancel_build",
            id_=cast("str", XComArg(create_build_without_wait, key="id")),
            project_id=PROJECT_ID,
        )

        retry_build = CloudBuildRetryBuildOperator(
            task_id="retry_build",
            id_=cast("str", XComArg(cancel_build, key="id")),
            project_id=PROJECT_ID,
        )

        get_build = CloudBuildGetBuildOperator(
            task_id="get_build",
            id_=cast("str", XComArg(retry_build, key="id")),
            project_id=PROJECT_ID,
        )

        create_build_without_wait >> cancel_build >> retry_build >> get_build

    # TEST BODY
    (
        [
            build_from_storage(),
            build_from_storage_deferrable(),
            build_from_repo(),
            build_from_repo_deferrable(),
            create_build_from_file,
            create_build_from_file_deferrable,
        ]
        >> list_builds
        >> [
            no_wait_cancel_retry_get(),
            no_wait_cancel_retry_get_deferrable(),
        ]
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG for Google AlloyDB operators.
"""

from __future__ import annotations

import os
from datetime import datetime

from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.alloy_db import (
    AlloyDBCreateBackupOperator,
    AlloyDBCreateClusterOperator,
    AlloyDBCreateInstanceOperator,
    AlloyDBCreateUserOperator,
    AlloyDBDeleteBackupOperator,
    AlloyDBDeleteClusterOperator,
    AlloyDBDeleteInstanceOperator,
    AlloyDBDeleteUserOperator,
    AlloyDBUpdateBackupOperator,
    AlloyDBUpdateClusterOperator,
    AlloyDBUpdateInstanceOperator,
    AlloyDBUpdateUserOperator,
)

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
DAG_ID = "alloy_db"
GCP_PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT", "default")

GCP_LOCATION = "europe-north1"
GCP_LOCATION_SECONDARY = "europe-west1"
GCP_LOCATION_BACKUP = "europe-west2"
GCP_NETWORK = "default"
CLUSTER_USER = "postgres-test"
CLUSTER_USER_PASSWORD = "postgres-test-pa$$w0rd"
CLUSTER_ID = f"cluster-{DAG_ID}-{ENV_ID}".replace("_", "-")
SECONDARY_CLUSTER_ID = f"cluster-secondary-{DAG_ID}-{ENV_ID}".replace("_", "-")
CLUSTER = {
    "network": f"projects/{GCP_PROJECT_ID}/global/networks/{GCP_NETWORK}",
    "initial_user": {
        "user": CLUSTER_USER,
        "password": CLUSTER_USER_PASSWORD,
    },
}
CLUSTER_UPDATE = {
    "automated_backup_policy": {
        "enabled": True,
    }
}
CLUSTER_UPDATE_MASK = {"paths": ["automated_backup_policy.enabled"]}
SECONDARY_CLUSTER = {
    "network": f"projects/{GCP_PROJECT_ID}/global/networks/{GCP_NETWORK}",
    "secondary_config": {
        "primary_cluster_name": f"projects/{GCP_PROJECT_ID}/locations/{GCP_LOCATION}/clusters/{CLUSTER_ID}",
    },
}
INSTANCE_ID = f"instance-{DAG_ID}-{ENV_ID}".replace("_", "-")
INSTANCE = {
    "instance_type": "PRIMARY",
}
INSTANCE_UPDATE = {"labels": {"label_test": "test_value"}}
INSTANCE_UPDATE_MASK = {"paths": ["labels"]}
SECONDARY_INSTANCE = {
    "instance_type": "SECONDARY",
}
SECONDARY_INSTANCE_ID = f"instance-secondary-{DAG_ID}-{ENV_ID}".replace("_", "-")
USER_ID = "test-user"
USER = {
    "password": "Test-Pa$$w0rd",
    "user_type": "ALLOYDB_BUILT_IN",
}
USER_UPDATE = {
    "database_roles": [
        "alloydbsuperuser",
    ]
}
USER_UPDATE_MASK = {
    "paths": ["database_roles"],
}
BACKUP_ID = "test-backup"
BACKUP = {
    "cluster_name": f"projects/{GCP_PROJECT_ID}/locations/{GCP_LOCATION}/clusters/{CLUSTER_ID}",
    "type": "ON_DEMAND",
}
BACKUP_UPDATE = {"labels": {"label_test": "test_value"}}
BACKUP_UPDATE_MASK = {
    "paths": ["labels"],
}

with DAG(
    DAG_ID,
    schedule="@once",  # Override to match your needs
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "alloy-db"],
) as dag:
    # [START howto_operator_alloy_db_create_cluster]
    create_cluster = AlloyDBCreateClusterOperator(
        task_id="create_cluster",
        cluster_id=CLUSTER_ID,
        cluster_configuration=CLUSTER,
        is_secondary=False,
        location=GCP_LOCATION,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_create_cluster]

    # [START howto_operator_alloy_db_update_cluster]
    update_cluster = AlloyDBUpdateClusterOperator(
        task_id="update_cluster",
        cluster_id=CLUSTER_ID,
        cluster_configuration=CLUSTER_UPDATE,
        update_mask=CLUSTER_UPDATE_MASK,
        location=GCP_LOCATION,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_update_cluster]

    # [START howto_operator_alloy_db_create_instance]
    create_instance = AlloyDBCreateInstanceOperator(
        task_id="create_instance",
        cluster_id=CLUSTER_ID,
        instance_id=INSTANCE_ID,
        instance_configuration=INSTANCE,
        is_secondary=False,
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
    )
    # [END howto_operator_alloy_db_create_instance]

    # [START howto_operator_alloy_db_create_backup]
    create_backup = AlloyDBCreateBackupOperator(
        task_id="create_backup",
        backup_id=BACKUP_ID,
        backup_configuration=BACKUP,
        location=GCP_LOCATION_BACKUP,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_create_backup]

    # [START howto_operator_alloy_db_update_backup]
    update_backup = AlloyDBUpdateBackupOperator(
        task_id="update_backup",
        backup_id=BACKUP_ID,
        backup_configuration=BACKUP_UPDATE,
        update_mask=BACKUP_UPDATE_MASK,
        location=GCP_LOCATION_BACKUP,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_update_backup]

    # [START howto_operator_alloy_db_update_instance]
    update_instance = AlloyDBUpdateInstanceOperator(
        task_id="update_instance",
        cluster_id=CLUSTER_ID,
        instance_id=INSTANCE_ID,
        instance_configuration=INSTANCE_UPDATE,
        update_mask=INSTANCE_UPDATE_MASK,
        location=GCP_LOCATION,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_update_instance]

    create_secondary_cluster = AlloyDBCreateClusterOperator(
        task_id="create_secondary_cluster",
        cluster_id=SECONDARY_CLUSTER_ID,
        cluster_configuration=SECONDARY_CLUSTER,
        is_secondary=True,
        location=GCP_LOCATION_SECONDARY,
        project_id=GCP_PROJECT_ID,
    )

    create_secondary_instance = AlloyDBCreateInstanceOperator(
        task_id="create_secondary_instance",
        cluster_id=SECONDARY_CLUSTER_ID,
        instance_id=SECONDARY_INSTANCE_ID,
        instance_configuration=SECONDARY_INSTANCE,
        is_secondary=True,
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION_SECONDARY,
    )

    # [START howto_operator_alloy_db_create_user]
    creat_user = AlloyDBCreateUserOperator(
        task_id="create_user",
        user_id=USER_ID,
        user_configuration=USER,
        cluster_id=CLUSTER_ID,
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
    )
    # [END howto_operator_alloy_db_create_user]

    # [START howto_operator_alloy_db_update_user]
    update_user = AlloyDBUpdateUserOperator(
        task_id="update_user",
        user_id=USER_ID,
        user_configuration=USER_UPDATE,
        cluster_id=CLUSTER_ID,
        update_mask=USER_UPDATE_MASK,
        location=GCP_LOCATION,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_update_user]

    # [START howto_operator_alloy_db_delete_user]
    delete_user = AlloyDBDeleteUserOperator(
        task_id="delete_user",
        cluster_id=CLUSTER_ID,
        user_id=USER_ID,
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
    )
    # [END howto_operator_alloy_db_delete_user]

    # [START howto_operator_alloy_db_delete_backup]
    delete_backup = AlloyDBDeleteBackupOperator(
        task_id="delete_backup",
        backup_id=BACKUP_ID,
        location=GCP_LOCATION_BACKUP,
        project_id=GCP_PROJECT_ID,
    )
    # [END howto_operator_alloy_db_delete_backup]
    delete_backup.trigger_rule = TriggerRule.ALL_DONE

    # [START howto_operator_alloy_db_delete_instance]
    delete_instance = AlloyDBDeleteInstanceOperator(
        task_id="delete_instance",
        cluster_id=CLUSTER_ID,
        instance_id=INSTANCE_ID,
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
    )
    # [END howto_operator_alloy_db_delete_instance]
    delete_instance.trigger_rule = TriggerRule.ALL_DONE

    delete_secondary_cluster = AlloyDBDeleteClusterOperator(
        task_id="delete_secondary_cluster",
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION_SECONDARY,
        cluster_id=SECONDARY_CLUSTER_ID,
        trigger_rule=TriggerRule.ALL_DONE,
        force=True,
    )

    # [START howto_operator_alloy_db_delete_cluster]
    delete_cluster = AlloyDBDeleteClusterOperator(
        task_id="delete_cluster",
        project_id=GCP_PROJECT_ID,
        location=GCP_LOCATION,
        cluster_id=CLUSTER_ID,
    )
    # [END howto_operator_alloy_db_delete_cluster]
    delete_cluster.trigger_rule = TriggerRule.ALL_DONE

    (
        create_cluster
        >> update_cluster
        >> create_instance
        >> update_instance
        >> create_backup
        >> update_backup
        >> create_secondary_cluster
        >> create_secondary_instance
        >> creat_user
        >> update_user
        >> delete_user
        >> delete_backup
        >> delete_secondary_cluster
        >> delete_instance
        >> delete_cluster
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "teardown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Airflow System Test DAG that verifies BigQueryToGCSOperator.
"""

from __future__ import annotations

import os
from datetime import datetime

from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.bigquery import (
    BigQueryCreateEmptyDatasetOperator,
    BigQueryCreateTableOperator,
    BigQueryDeleteDatasetOperator,
)
from airflow.providers.google.cloud.operators.gcs import GCSCreateBucketOperator, GCSDeleteBucketOperator
from airflow.providers.google.cloud.transfers.bigquery_to_gcs import BigQueryToGCSOperator

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
DAG_ID = "bigquery_to_gcs"

DATASET_NAME = f"dataset_{DAG_ID}_{ENV_ID}"
BUCKET_NAME = f"bucket_{DAG_ID}_{ENV_ID}"
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
BUCKET_FILE = "test.csv"
TABLE = "test"
SCHEMA = [
    {"name": "emp_name", "type": "STRING", "mode": "REQUIRED"},
    {"name": "salary", "type": "INTEGER", "mode": "NULLABLE"},
]


with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "bigquery"],
) as dag:
    create_bucket = GCSCreateBucketOperator(
        task_id="create_bucket", bucket_name=BUCKET_NAME, project_id=PROJECT_ID
    )

    create_dataset = BigQueryCreateEmptyDatasetOperator(task_id="create_dataset", dataset_id=DATASET_NAME)

    create_table = BigQueryCreateTableOperator(
        task_id="create_table",
        dataset_id=DATASET_NAME,
        table_id=TABLE,
        table_resource={
            "schema": {"fields": SCHEMA},
        },
    )

    # [START howto_operator_bigquery_to_gcs]
    bigquery_to_gcs = BigQueryToGCSOperator(
        task_id="bigquery_to_gcs",
        source_project_dataset_table=f"{DATASET_NAME}.{TABLE}",
        destination_cloud_storage_uris=[f"gs://{BUCKET_NAME}/{BUCKET_FILE}"],
    )
    # [END howto_operator_bigquery_to_gcs]

    bigquery_to_gcs_async = BigQueryToGCSOperator(
        task_id="bigquery_to_gcs_async",
        source_project_dataset_table=f"{DATASET_NAME}.{TABLE}",
        destination_cloud_storage_uris=[f"gs://{BUCKET_NAME}/{BUCKET_FILE}"],
        deferrable=True,
    )

    delete_bucket = GCSDeleteBucketOperator(
        task_id="delete_bucket", bucket_name=BUCKET_NAME, trigger_rule=TriggerRule.ALL_DONE
    )

    delete_dataset = BigQueryDeleteDatasetOperator(
        task_id="delete_dataset",
        dataset_id=DATASET_NAME,
        delete_contents=True,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    (
        # TEST SETUP
        [create_bucket, create_dataset]
        >> create_table
        # TEST BODY
        >> bigquery_to_gcs
        >> bigquery_to_gcs_async
        # TEST TEARDOWN
        >> [delete_bucket, delete_dataset]
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

from airflow.exceptions import AirflowException, AirflowProviderDeprecationWarning
from airflow.providers.google.marketing_platform.sensors.display_video import (
    GoogleDisplayVideo360GetSDFDownloadOperationSensor,
    GoogleDisplayVideo360RunQuerySensor,
)

MODULE_NAME = "airflow.providers.google.marketing_platform.sensors.display_video"

API_VERSION = "api_version"
GCP_CONN_ID = "google_cloud_default"


class TestGoogleDisplayVideo360RunQuerySensor:
    @mock.patch(f"{MODULE_NAME}.GoogleDisplayVideo360Hook")
    @mock.patch(f"{MODULE_NAME}.BaseSensorOperator")
    def test_poke(self, mock_base_op, hook_mock):
        query_id = "QUERY_ID"
        report_id = "REPORT_ID"
        with pytest.warns(AirflowProviderDeprecationWarning):  # noqa: PT031
            op = GoogleDisplayVideo360RunQuerySensor(
                query_id=query_id, report_id=report_id, task_id="test_task"
            )
            op.poke(context=None)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version="v2",
            impersonation_chain=None,
        )
        hook_mock.return_value.get_report.assert_called_once_with(query_id=query_id, report_id=report_id)


class TestGoogleDisplayVideo360Sensor:
    @mock.patch(f"{MODULE_NAME}.GoogleDisplayVideo360Hook")
    @mock.patch(f"{MODULE_NAME}.BaseSensorOperator")
    def test_poke(self, mock_base_op, hook_mock):
        operation_name = "operation_name"
        op = GoogleDisplayVideo360GetSDFDownloadOperationSensor(
            operation_name=operation_name,
            api_version=API_VERSION,
            task_id="test_task",
        )
        op.poke(context=None)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=None,
        )
        hook_mock.return_value.get_sdf_download_operation.assert_called_once_with(
            operation_name=operation_name
        )

    @mock.patch(f"{MODULE_NAME}.GoogleDisplayVideo360Hook")
    @mock.patch(f"{MODULE_NAME}.BaseSensorOperator")
    def test_poke_with_exception(
        self,
        mock_base_op,
        hook_mock,
    ):
        operation_name = "operation_name"
        op = GoogleDisplayVideo360GetSDFDownloadOperationSensor(
            operation_name=operation_name,
            api_version=API_VERSION,
            task_id="test_task",
        )
        hook_mock.return_value.get_sdf_download_operation.return_value = {"error": "error"}

        with pytest.raises(AirflowException, match="The operation finished in error with error"):
            op.poke(context={})



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import logging
from unittest import mock

import pytest
from google.api_core.gapic_v1.method import DEFAULT
from google.cloud.dataform_v1beta1.types import WorkflowInvocation

from airflow.exceptions import AirflowException
from airflow.providers.google.cloud.hooks.dataform import DataformHook

from unit.google.cloud.utils.base_gcp_mock import mock_base_gcp_hook_default_project_id

BASE_STRING = "airflow.providers.google.common.hooks.base_google.{}"
DATAFORM_STRING = "airflow.providers.google.cloud.hooks.dataform.{}"

PROJECT_ID = "project-id"
REGION = "region"
REPOSITORY_ID = "test_repository"
WORKSPACE_ID = "test_workspace"
GCP_CONN_ID = "google_cloud_default"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]
COMPILATION_RESULT = {
    "git_commitish": "main",
    "workspace": (
        f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
    ),
}
COMPILATION_RESULT_ID = "test_compilation_result_id"
WORKFLOW_INVOCATION = {
    "compilation_result": (
        f"projects/{PROJECT_ID}/locations/{REGION}/repositories/"
        f"{REPOSITORY_ID}/compilationResults/{COMPILATION_RESULT_ID}"
    ),
}
WORKFLOW_INVOCATION_ID = "test_workflow_invocation_id"
PATH_TO_FOLDER = "path/to/folder"
FILEPATH = "path/to/file.txt"
FILE_CONTENTS = b"test content"


class TestDataformHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"),
            new=mock_base_gcp_hook_default_project_id,
        ):
            self.hook = DataformHook(
                gcp_conn_id=GCP_CONN_ID,
                impersonation_chain=IMPERSONATION_CHAIN,
            )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_create_compilation_result(self, mock_client):
        self.hook.create_compilation_result(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            compilation_result=COMPILATION_RESULT,
        )
        parent = f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}"
        mock_client.return_value.create_compilation_result.assert_called_once_with(
            request=dict(parent=parent, compilation_result=COMPILATION_RESULT),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_compilation_result"))
    def get_compilation_result(self, mock_client):
        self.hook.create_compilation_result(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
        )
        name = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/"
            f"{REPOSITORY_ID}/compilationResults/{COMPILATION_RESULT_ID}"
        )
        mock_client.return_value.get_compilation_result.assert_called_once_with(
            request=dict(
                name=name,
            ),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_create_workflow_invocation(self, mock_client):
        self.hook.create_workflow_invocation(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workflow_invocation=WORKFLOW_INVOCATION,
        )
        parent = f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}"
        mock_client.return_value.create_workflow_invocation.assert_called_once_with(
            request=dict(parent=parent, workflow_invocation=WORKFLOW_INVOCATION),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_get_workflow_invocation(self, mock_client):
        self.hook.get_workflow_invocation(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workflow_invocation_id=WORKFLOW_INVOCATION_ID,
        )
        name = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/"
            f"{REPOSITORY_ID}/workflowInvocations/{WORKFLOW_INVOCATION_ID}"
        )
        mock_client.return_value.get_workflow_invocation.assert_called_once_with(
            request=dict(
                name=name,
            ),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_query_workflow_invocation_actions(self, mock_client):
        self.hook.query_workflow_invocation_actions(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workflow_invocation_id=WORKFLOW_INVOCATION_ID,
        )
        name = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/"
            f"{REPOSITORY_ID}/workflowInvocations/{WORKFLOW_INVOCATION_ID}"
        )
        mock_client.return_value.query_workflow_invocation_actions.assert_called_once_with(
            request=dict(
                name=name,
            ),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_workflow_invocation"))
    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_cancel_workflow_invocation(self, mock_client, mock_state):
        mock_state.return_value.state = WorkflowInvocation.State.RUNNING
        name = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/"
            f"{REPOSITORY_ID}/workflowInvocations/{WORKFLOW_INVOCATION_ID}"
        )

        self.hook.cancel_workflow_invocation(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workflow_invocation_id=WORKFLOW_INVOCATION_ID,
        )
        mock_client.return_value.cancel_workflow_invocation.assert_called_once_with(
            request=dict(
                name=name,
            ),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_workflow_invocation"))
    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_get_workflow_invocation_raises_exception_on_cancel_workflow_invocation(
        self, mock_client, mock_state
    ):
        mock_client.return_value.get_dataform_client.return_value = None
        mock_state.side_effect = AirflowException(
            "Dataform API returned error when waiting for workflow invocation"
        )

        with pytest.raises(AirflowException, match="Dataform API returned error*."):
            self.hook.cancel_workflow_invocation(
                project_id=PROJECT_ID,
                region=REGION,
                repository_id=REPOSITORY_ID,
                workflow_invocation_id=WORKFLOW_INVOCATION_ID,
            )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_workflow_invocation"))
    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_cancel_workflow_invocation_is_not_called(self, mock_client, mock_state, caplog):
        mock_state.return_value.state = WorkflowInvocation.State.SUCCEEDED
        expected_log = (
            "Workflow is not active. Either the execution has already "
            "finished or has been canceled. Please check the logs above "
            "for more details."
        )

        with caplog.at_level(logging.INFO):
            self.hook.cancel_workflow_invocation(
                project_id=PROJECT_ID,
                region=REGION,
                repository_id=REPOSITORY_ID,
                workflow_invocation_id=WORKFLOW_INVOCATION_ID,
            )
            assert expected_log in caplog.text

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_create_repository(self, mock_client):
        self.hook.create_repository(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
        )
        parent = f"projects/{PROJECT_ID}/locations/{REGION}"
        mock_client.return_value.create_repository.assert_called_once_with(
            request={"parent": parent, "repository_id": REPOSITORY_ID},
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_create_workspace(self, mock_client):
        self.hook.create_workspace(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
        )
        parent = f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}"

        mock_client.return_value.create_workspace.assert_called_once_with(
            request={
                "parent": parent,
                "workspace_id": WORKSPACE_ID,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_delete_repository(self, mock_client):
        force = True
        self.hook.delete_repository(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            force=force,
        )
        name = f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}"

        mock_client.return_value.delete_repository.assert_called_once_with(
            request={
                "name": name,
                "force": force,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_delete_workspace(self, mock_client):
        self.hook.delete_workspace(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
        )
        name = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
        )

        mock_client.return_value.delete_workspace.assert_called_once_with(
            request={
                "name": name,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_write_file(self, mock_client):
        self.hook.write_file(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
            filepath=FILEPATH,
            contents=FILE_CONTENTS,
        )
        workspace_path = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
        )

        mock_client.return_value.write_file.assert_called_once_with(
            request={
                "workspace": workspace_path,
                "path": FILEPATH,
                "contents": FILE_CONTENTS,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_make_directory(self, mock_client):
        self.hook.make_directory(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
            path=PATH_TO_FOLDER,
        )
        workspace_path = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
        )

        mock_client.return_value.make_directory.assert_called_once_with(
            request={
                "workspace": workspace_path,
                "path": PATH_TO_FOLDER,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_remove_directory(self, mock_client):
        self.hook.remove_directory(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
            path=PATH_TO_FOLDER,
        )
        workspace_path = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
        )

        mock_client.return_value.remove_directory.assert_called_once_with(
            request={
                "workspace": workspace_path,
                "path": PATH_TO_FOLDER,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_remove_file(self, mock_client):
        self.hook.remove_file(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
            filepath=FILEPATH,
        )
        workspace_path = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
        )

        mock_client.return_value.remove_file.assert_called_once_with(
            request={
                "workspace": workspace_path,
                "path": FILEPATH,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch(DATAFORM_STRING.format("DataformHook.get_dataform_client"))
    def test_install_npm_packages(self, mock_client):
        self.hook.install_npm_packages(
            project_id=PROJECT_ID,
            region=REGION,
            repository_id=REPOSITORY_ID,
            workspace_id=WORKSPACE_ID,
        )

        workspace_path = (
            f"projects/{PROJECT_ID}/locations/{REGION}/repositories/{REPOSITORY_ID}/workspaces/{WORKSPACE_ID}"
        )

        mock_client.return_value.install_npm_packages.assert_called_once_with(
            request={
                "workspace": workspace_path,
            },
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example Airflow DAG for Google BigQuery service testing tables.
"""

from __future__ import annotations

import os
import time
from datetime import datetime
from pathlib import Path

from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.bigquery import (
    BigQueryCreateEmptyDatasetOperator,
    BigQueryCreateTableOperator,
    BigQueryDeleteDatasetOperator,
    BigQueryDeleteTableOperator,
    BigQueryGetDatasetTablesOperator,
    BigQueryUpdateDatasetOperator,
    BigQueryUpdateTableOperator,
    BigQueryUpdateTableSchemaOperator,
    BigQueryUpsertTableOperator,
)
from airflow.providers.google.cloud.operators.gcs import GCSCreateBucketOperator, GCSDeleteBucketOperator
from airflow.providers.google.cloud.transfers.local_to_gcs import LocalFilesystemToGCSOperator

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
from system.openlineage.operator import OpenLineageTestOperator

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
DAG_ID = "bigquery_tables"

BUCKET_NAME = f"bucket_{DAG_ID}_{ENV_ID}"
DATASET_NAME = f"dataset_{DAG_ID}_{ENV_ID}"
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
SCHEMA_JSON_LOCAL_SRC = str(Path(__file__).parent / "resources" / "update_table_schema.json")
SCHEMA_JSON_DESTINATION = "update_table_schema.json"
GCS_PATH_TO_SCHEMA_JSON = f"gs://{BUCKET_NAME}/{SCHEMA_JSON_DESTINATION}"


with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "bigquery"],
) as dag:
    create_bucket = GCSCreateBucketOperator(
        task_id="create_bucket", bucket_name=BUCKET_NAME, project_id=PROJECT_ID
    )

    upload_schema_json = LocalFilesystemToGCSOperator(
        task_id="upload_schema_json",
        src=SCHEMA_JSON_LOCAL_SRC,
        dst=SCHEMA_JSON_DESTINATION,
        bucket=BUCKET_NAME,
    )

    create_dataset = BigQueryCreateEmptyDatasetOperator(task_id="create_dataset", dataset_id=DATASET_NAME)

    # [START howto_operator_bigquery_create_table]
    create_table = BigQueryCreateTableOperator(
        task_id="create_table",
        dataset_id=DATASET_NAME,
        table_id="test_table",
        table_resource={
            "schema": {
                "fields": [
                    {"name": "emp_name", "type": "STRING", "mode": "REQUIRED"},
                    {"name": "salary", "type": "INTEGER", "mode": "NULLABLE"},
                ],
            },
        },
    )
    # [END howto_operator_bigquery_create_table]

    # [START howto_operator_bigquery_create_view]
    create_view = BigQueryCreateTableOperator(
        task_id="create_view",
        dataset_id=DATASET_NAME,
        table_id="test_view",
        table_resource={
            "view": {
                "query": f"SELECT * FROM `{PROJECT_ID}.{DATASET_NAME}.test_table`",
                "useLegacySql": False,
            },
        },
    )
    # [END howto_operator_bigquery_create_view]

    # [START howto_operator_bigquery_create_materialized_view]
    create_materialized_view = BigQueryCreateTableOperator(
        task_id="create_materialized_view",
        dataset_id=DATASET_NAME,
        table_id="test_materialized_view",
        table_resource={
            "materializedView": {
                "query": f"SELECT SUM(salary) AS sum_salary FROM `{PROJECT_ID}.{DATASET_NAME}.test_table`",
                "enableRefresh": True,
                "refreshIntervalMs": 600000,
            },
        },
    )
    # [END howto_operator_bigquery_create_materialized_view]

    # [START howto_operator_bigquery_delete_view]
    delete_view = BigQueryDeleteTableOperator(
        task_id="delete_view",
        deletion_dataset_table=f"{PROJECT_ID}.{DATASET_NAME}.test_view",
    )
    # [END howto_operator_bigquery_delete_view]

    # [START howto_operator_bigquery_update_table]
    update_table = BigQueryUpdateTableOperator(
        task_id="update_table",
        dataset_id=DATASET_NAME,
        table_id="test_table",
        fields=["friendlyName", "description"],
        table_resource={
            "friendlyName": "Updated Table",
            "description": "Updated Table",
        },
    )
    # [END howto_operator_bigquery_update_table]

    # [START howto_operator_bigquery_update_view]
    update_view = BigQueryUpdateTableOperator(
        task_id="update_view",
        dataset_id=DATASET_NAME,
        table_id="test_view",
        fields=["friendlyName", "description"],
        table_resource={
            "friendlyName": "Updated View friendlyName",
            "description": "Updated View description",
        },
    )
    # [END howto_operator_bigquery_update_view]

    # [START howto_operator_bigquery_update_materialized_view]
    update_materialized_view = BigQueryUpdateTableOperator(
        task_id="update_materialized_view",
        dataset_id=DATASET_NAME,
        table_id="test_materialized_view",
        fields=["friendlyName", "description"],
        table_resource={
            "friendlyName": "Updated View friendlyName",
            "description": "Updated View description",
        },
    )
    # [END howto_operator_bigquery_update_materialized_view]

    # [START howto_operator_bigquery_upsert_table]
    upsert_table = BigQueryUpsertTableOperator(
        task_id="upsert_table",
        dataset_id=DATASET_NAME,
        table_resource={
            "tableReference": {"tableId": "test_table_id"},
            "expirationTime": (int(time.time()) + 300) * 1000,
        },
    )
    # [END howto_operator_bigquery_upsert_table]

    # [START howto_operator_bigquery_update_table_schema]
    update_table_schema = BigQueryUpdateTableSchemaOperator(
        task_id="update_table_schema",
        dataset_id=DATASET_NAME,
        table_id="test_table",
        schema_fields_updates=[
            {"name": "emp_name", "description": "Name of employee"},
            {"name": "salary", "description": "Monthly salary in USD"},
        ],
    )
    # [END howto_operator_bigquery_update_table_schema]

    # [START howto_operator_bigquery_create_table_schema_json]
    create_table_schema_json = BigQueryCreateTableOperator(
        task_id="create_table_schema_json",
        dataset_id=DATASET_NAME,
        table_id="test_table",
        gcs_schema_object=GCS_PATH_TO_SCHEMA_JSON,
        table_resource={
            "tableReference": {
                "projectId": PROJECT_ID,
                "datasetId": DATASET_NAME,
                "tableId": "test_table",
            },
        },
    )
    # [END howto_operator_bigquery_create_table_schema_json]

    # [START howto_operator_bigquery_delete_materialized_view]
    delete_materialized_view = BigQueryDeleteTableOperator(
        task_id="delete_materialized_view",
        deletion_dataset_table=f"{PROJECT_ID}.{DATASET_NAME}.test_materialized_view",
    )
    # [END howto_operator_bigquery_delete_materialized_view]

    # [START howto_operator_bigquery_get_dataset_tables]
    get_dataset_tables = BigQueryGetDatasetTablesOperator(
        task_id="get_dataset_tables", dataset_id=DATASET_NAME
    )
    # [END howto_operator_bigquery_get_dataset_tables]

    update_dataset = BigQueryUpdateDatasetOperator(
        task_id="update_dataset",
        dataset_id=DATASET_NAME,
        dataset_resource={"description": "Updated dataset"},
    )

    # [START howto_operator_bigquery_delete_table]
    delete_table = BigQueryDeleteTableOperator(
        task_id="delete_table",
        deletion_dataset_table=f"{PROJECT_ID}.{DATASET_NAME}.test_table",
    )
    # [END howto_operator_bigquery_delete_table]

    delete_dataset = BigQueryDeleteDatasetOperator(
        task_id="delete_dataset", dataset_id=DATASET_NAME, delete_contents=True
    )
    delete_dataset.trigger_rule = TriggerRule.ALL_DONE

    delete_bucket = GCSDeleteBucketOperator(
        task_id="delete_bucket", bucket_name=BUCKET_NAME, trigger_rule=TriggerRule.ALL_DONE
    )

    check_openlineage_events = OpenLineageTestOperator(
        task_id="check_openlineage_events",
        file_path=str(Path(__file__).parent / "resources" / "openlineage" / "bigquery_tables.json"),
    )

    (
        # TEST SETUP
        create_bucket
        >> create_dataset
        >> upload_schema_json
        # TEST BODY
        >> update_dataset
        >> create_table
        >> create_view
        >> create_materialized_view
        >> update_view
        >> [
            get_dataset_tables,
            delete_view,
        ]
        >> update_table
        >> update_materialized_view
        >> upsert_table
        >> update_table_schema
        >> create_table_schema_json
        >> delete_materialized_view
        >> delete_table
        # TEST TEARDOWN
        >> delete_bucket
        >> delete_dataset
        >> check_openlineage_events
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from airflow.providers.google.cloud.links.base import BaseGoogleLink

BASE_LINK = "https://clouddataprep.com"
DATAPREP_FLOW_LINK = BASE_LINK + "/flows/{flow_id}?projectId={project_id}"
DATAPREP_JOB_GROUP_LINK = BASE_LINK + "/jobs/{job_group_id}?projectId={project_id}"


class DataprepFlowLink(BaseGoogleLink):
    """Helper class for constructing Dataprep flow link."""

    name = "Flow details page"
    key = "dataprep_flow_page"
    format_str = DATAPREP_FLOW_LINK


class DataprepJobGroupLink(BaseGoogleLink):
    """Helper class for constructing Dataprep job group link."""

    name = "Job group details page"
    key = "dataprep_job_group_page"
    format_str = DATAPREP_JOB_GROUP_LINK



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from google.api_core.gapic_v1.method import DEFAULT
from google.cloud.automl_v1beta1 import AutoMlClient

from airflow.providers.google.cloud.hooks.automl import CloudAutoMLHook
from airflow.providers.google.common.consts import CLIENT_INFO

from unit.google.cloud.utils.base_gcp_mock import mock_base_gcp_hook_no_default_project_id

CREDENTIALS = "test-creds"
TASK_ID = "test-automl-hook"
GCP_PROJECT_ID = "test-project"
GCP_LOCATION = "test-location"
MODEL_NAME = "test_model"
MODEL_ID = "projects/198907790164/locations/us-central1/models/TBL9195602771183665152"
DATASET_ID = "TBL123456789"
MODEL = {
    "display_name": MODEL_NAME,
    "dataset_id": DATASET_ID,
    "tables_model_metadata": {"train_budget_milli_node_hours": 1000},
}

LOCATION_PATH = f"projects/{GCP_PROJECT_ID}/locations/{GCP_LOCATION}"
MODEL_PATH = f"projects/{GCP_PROJECT_ID}/locations/{GCP_LOCATION}/models/{MODEL_ID}"
DATASET_PATH = f"projects/{GCP_PROJECT_ID}/locations/{GCP_LOCATION}/datasets/{DATASET_ID}"

INPUT_CONFIG = {"input": "value"}
OUTPUT_CONFIG = {"output": "value"}
PAYLOAD = {"test": "payload"}
DATASET = {"dataset_id": "data"}
MASK = {"field": "mask"}


class TestAutoMLHook:
    def setup_method(self):
        with mock.patch(
            "airflow.providers.google.cloud.hooks.automl.GoogleBaseHook.__init__",
            new=mock_base_gcp_hook_no_default_project_id,
        ):
            self.hook = CloudAutoMLHook()
            self.hook.get_credentials = mock.MagicMock(return_value=CREDENTIALS)

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient")
    def test_get_conn(self, mock_automl_client):
        self.hook.get_conn()
        mock_automl_client.assert_called_once_with(credentials=CREDENTIALS, client_info=CLIENT_INFO)

    @mock.patch("airflow.providers.google.cloud.hooks.automl.PredictionServiceClient")
    def test_prediction_client(self, mock_prediction_client):
        client = self.hook.prediction_client  # noqa: F841
        mock_prediction_client.assert_called_once_with(credentials=CREDENTIALS, client_info=CLIENT_INFO)

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.create_model")
    def test_create_model(self, mock_create_model):
        self.hook.create_model(model=MODEL, location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_create_model.assert_called_once_with(
            request=dict(parent=LOCATION_PATH, model=MODEL), retry=DEFAULT, timeout=None, metadata=()
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.PredictionServiceClient.batch_predict")
    def test_batch_predict(self, mock_batch_predict):
        self.hook.batch_predict(
            model_id=MODEL_ID,
            location=GCP_LOCATION,
            project_id=GCP_PROJECT_ID,
            input_config=INPUT_CONFIG,
            output_config=OUTPUT_CONFIG,
        )

        mock_batch_predict.assert_called_once_with(
            request=dict(
                name=MODEL_PATH, input_config=INPUT_CONFIG, output_config=OUTPUT_CONFIG, params=None
            ),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.PredictionServiceClient.predict")
    def test_predict(self, mock_predict):
        self.hook.predict(
            model_id=MODEL_ID,
            location=GCP_LOCATION,
            project_id=GCP_PROJECT_ID,
            payload=PAYLOAD,
        )

        mock_predict.assert_called_once_with(
            request=dict(name=MODEL_PATH, payload=PAYLOAD, params=None),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.create_dataset")
    def test_create_dataset(self, mock_create_dataset):
        self.hook.create_dataset(dataset=DATASET, location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_create_dataset.assert_called_once_with(
            request=dict(parent=LOCATION_PATH, dataset=DATASET),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.import_data")
    def test_import_dataset(self, mock_import_data):
        self.hook.import_data(
            dataset_id=DATASET_ID,
            location=GCP_LOCATION,
            project_id=GCP_PROJECT_ID,
            input_config=INPUT_CONFIG,
        )

        mock_import_data.assert_called_once_with(
            request=dict(name=DATASET_PATH, input_config=INPUT_CONFIG),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.list_column_specs")
    def test_list_column_specs(self, mock_list_column_specs):
        table_spec = "table_spec_id"
        filter_ = "filter"
        page_size = 42

        self.hook.list_column_specs(
            dataset_id=DATASET_ID,
            table_spec_id=table_spec,
            location=GCP_LOCATION,
            project_id=GCP_PROJECT_ID,
            field_mask=MASK,
            filter_=filter_,
            page_size=page_size,
        )

        parent = AutoMlClient.table_spec_path(GCP_PROJECT_ID, GCP_LOCATION, DATASET_ID, table_spec)
        mock_list_column_specs.assert_called_once_with(
            request=dict(parent=parent, field_mask=MASK, filter=filter_, page_size=page_size),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.get_model")
    def test_get_model(self, mock_get_model):
        self.hook.get_model(model_id=MODEL_ID, location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_get_model.assert_called_once_with(
            request=dict(name=MODEL_PATH), retry=DEFAULT, timeout=None, metadata=()
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.delete_model")
    def test_delete_model(self, mock_delete_model):
        self.hook.delete_model(model_id=MODEL_ID, location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_delete_model.assert_called_once_with(
            request=dict(name=MODEL_PATH), retry=DEFAULT, timeout=None, metadata=()
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.update_dataset")
    def test_update_dataset(self, mock_update_dataset):
        self.hook.update_dataset(
            dataset=DATASET,
            update_mask=MASK,
        )

        mock_update_dataset.assert_called_once_with(
            request=dict(dataset=DATASET, update_mask=MASK), retry=DEFAULT, timeout=None, metadata=()
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.deploy_model")
    def test_deploy_model(self, mock_deploy_model):
        image_detection_metadata = {}

        self.hook.deploy_model(
            model_id=MODEL_ID,
            image_detection_metadata=image_detection_metadata,
            location=GCP_LOCATION,
            project_id=GCP_PROJECT_ID,
        )

        mock_deploy_model.assert_called_once_with(
            request=dict(
                name=MODEL_PATH,
                image_object_detection_model_deployment_metadata=image_detection_metadata,
            ),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.list_table_specs")
    def test_list_table_specs(self, mock_list_table_specs):
        filter_ = "filter"
        page_size = 42

        self.hook.list_table_specs(
            dataset_id=DATASET_ID,
            location=GCP_LOCATION,
            project_id=GCP_PROJECT_ID,
            filter_=filter_,
            page_size=page_size,
        )

        mock_list_table_specs.assert_called_once_with(
            request=dict(parent=DATASET_PATH, filter=filter_, page_size=page_size),
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.list_datasets")
    def test_list_datasets(self, mock_list_datasets):
        self.hook.list_datasets(location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_list_datasets.assert_called_once_with(
            request=dict(parent=LOCATION_PATH), retry=DEFAULT, timeout=None, metadata=()
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.delete_dataset")
    def test_delete_dataset(self, mock_delete_dataset):
        self.hook.delete_dataset(dataset_id=DATASET_ID, location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_delete_dataset.assert_called_once_with(
            request=dict(name=DATASET_PATH), retry=DEFAULT, timeout=None, metadata=()
        )

    @mock.patch("airflow.providers.google.cloud.hooks.automl.AutoMlClient.get_dataset")
    def test_get_dataset(self, mock_get_dataset):
        self.hook.get_dataset(dataset_id=DATASET_ID, location=GCP_LOCATION, project_id=GCP_PROJECT_ID)

        mock_get_dataset.assert_called_once_with(
            request=dict(name=DATASET_PATH), retry=DEFAULT, timeout=None, metadata=()
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Example Airflow DAG for Google BigQuery service.

The DAG checks how BigQueryValueCheckOperator works with a non-US dataset.
"""

from __future__ import annotations

import os
from datetime import datetime, timedelta

from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.bigquery import (
    BigQueryCreateEmptyDatasetOperator,
    BigQueryCreateTableOperator,
    BigQueryDeleteDatasetOperator,
    BigQueryInsertJobOperator,
    BigQueryValueCheckOperator,
)

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT", "default")
NON_US_LOCATION = "asia-east1"

SCHEMA = [
    {"name": "value", "type": "INTEGER", "mode": "REQUIRED"},
    {"name": "name", "type": "STRING", "mode": "NULLABLE"},
    {"name": "ds", "type": "DATE", "mode": "NULLABLE"},
]

DAG_ID = "bq_value_check_location"
DATASET = f"ds_{DAG_ID}_{ENV_ID}"
TABLE = "ds_table"
INSERT_DATE = datetime.now().strftime("%Y-%m-%d")
INSERT_ROWS_QUERY = (
    f"INSERT {DATASET}.{TABLE} VALUES "
    f"(42, 'monty python', '{INSERT_DATE}'), "
    f"(42, 'fishy fish', '{INSERT_DATE}');"
)
default_args = {
    "execution_timeout": timedelta(minutes=10),
    "retries": 2,
    "retry_delay": timedelta(seconds=30),
}

with DAG(
    DAG_ID,
    schedule="@once",
    catchup=False,
    start_date=datetime(2024, 1, 1),
    default_args=default_args,
    tags=["example", "bigquery"],
) as dag:
    create_dataset = BigQueryCreateEmptyDatasetOperator(
        task_id="create_dataset",
        dataset_id=DATASET,
        location=NON_US_LOCATION,
    )

    create_table = BigQueryCreateTableOperator(
        task_id="create_table",
        dataset_id=DATASET,
        table_id=TABLE,
        table_resource={
            "schema": {"fields": SCHEMA},
        },
    )

    insert_query_job = BigQueryInsertJobOperator(
        task_id="insert_query_job",
        configuration={
            "query": {
                "query": INSERT_ROWS_QUERY,
                "useLegacySql": False,
                "priority": "BATCH",
            }
        },
        location=NON_US_LOCATION,
    )

    check_value = BigQueryValueCheckOperator(
        task_id="check_value",
        sql=f"SELECT COUNT(*) FROM {DATASET}.{TABLE}",
        pass_value=2,
        use_legacy_sql=False,
        location=NON_US_LOCATION,
    )

    check_value_no_location = BigQueryValueCheckOperator(
        task_id="check_value_no_location",
        sql=f"SELECT COUNT(*) FROM {DATASET}.{TABLE}",
        pass_value=2,
        use_legacy_sql=False,
        deferrable=False,
    )

    delete_dataset = BigQueryDeleteDatasetOperator(
        task_id="delete_dataset",
        dataset_id=DATASET,
        delete_contents=True,
        trigger_rule=TriggerRule.ALL_DONE,
    )

    (
        # TEST SETUP
        create_dataset
        >> create_table
        >> insert_query_job
        # TEST BODY
        >> check_value
        >> check_value_no_location
        # TEST TEARDOWN
        >> delete_dataset
    )

    from tests_common.test_utils.system_tests import get_test_run
    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

    # Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
    test_run = get_test_run(dag)


from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from tempfile import NamedTemporaryFile
from unittest import mock

import pytest

from airflow.models import TaskInstance as TI
from airflow.providers.google.marketing_platform.operators.campaign_manager import (
    GoogleCampaignManagerBatchInsertConversionsOperator,
    GoogleCampaignManagerBatchUpdateConversionsOperator,
    GoogleCampaignManagerDeleteReportOperator,
    GoogleCampaignManagerDownloadReportOperator,
    GoogleCampaignManagerInsertReportOperator,
    GoogleCampaignManagerRunReportOperator,
)
from airflow.utils import timezone
from airflow.utils.session import create_session

API_VERSION = "v4"
GCP_CONN_ID = "google_cloud_default"

CONVERSION = {
    "kind": "dfareporting#conversion",
    "floodlightActivityId": 1234,
    "floodlightConfigurationId": 1234,
    "gclid": "971nc2849184c1914019v1c34c14",
    "ordinal": "0",
    "customVariables": [
        {
            "kind": "dfareporting#customFloodlightVariable",
            "type": "U10",
            "value": "value",
        }
    ],
}

DEFAULT_DATE = timezone.datetime(2021, 1, 1)
PROFILE_ID = "profile_id"
REPORT_ID = "report_id"
FILE_ID = "file_id"
BUCKET_NAME = "test_bucket"
REPORT_NAME = "test_report.csv"
TEMP_FILE_NAME = "test"


class TestGoogleCampaignManagerDeleteReportOperator:
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.BaseOperator")
    def test_execute(self, mock_base_op, hook_mock):
        op = GoogleCampaignManagerDeleteReportOperator(
            profile_id=PROFILE_ID,
            report_id=REPORT_ID,
            api_version=API_VERSION,
            task_id="test_task",
        )
        op.execute(context=None)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=None,
        )
        hook_mock.return_value.delete_report.assert_called_once_with(
            profile_id=PROFILE_ID, report_id=REPORT_ID
        )


@pytest.mark.db_test
class TestGoogleCampaignManagerDownloadReportOperator:
    def setup_method(self):
        with create_session() as session:
            session.query(TI).delete()

    def teardown_method(self):
        with create_session() as session:
            session.query(TI).delete()

    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.http")
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.tempfile")
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.GCSHook")
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.BaseOperator")
    def test_execute(
        self,
        mock_base_op,
        gcs_hook_mock,
        hook_mock,
        tempfile_mock,
        http_mock,
    ):
        http_mock.MediaIoBaseDownload.return_value.next_chunk.return_value = (
            None,
            True,
        )
        tempfile_mock.NamedTemporaryFile.return_value.__enter__.return_value.name = TEMP_FILE_NAME

        mock_context = {"task_instance": mock.Mock()}

        op = GoogleCampaignManagerDownloadReportOperator(
            profile_id=PROFILE_ID,
            report_id=REPORT_ID,
            file_id=FILE_ID,
            bucket_name=BUCKET_NAME,
            report_name=REPORT_NAME,
            api_version=API_VERSION,
            task_id="test_task",
        )
        op.execute(context=mock_context)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=None,
        )
        hook_mock.return_value.get_report_file.assert_called_once_with(
            profile_id=PROFILE_ID, report_id=REPORT_ID, file_id=FILE_ID
        )
        gcs_hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            impersonation_chain=None,
        )
        gcs_hook_mock.return_value.upload.assert_called_once_with(
            bucket_name=BUCKET_NAME,
            object_name=REPORT_NAME + ".gz",
            gzip=True,
            filename=TEMP_FILE_NAME,
            mime_type="text/csv",
        )
        mock_context["task_instance"].xcom_push.assert_called_once_with(
            key="report_name", value=REPORT_NAME + ".gz"
        )

    @pytest.mark.parametrize(
        "test_bucket_name",
        [BUCKET_NAME, f"gs://{BUCKET_NAME}", "XComArg", "{{ ti.xcom_pull(task_ids='taskflow_op') }}"],
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.http")
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.tempfile")
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.GCSHook")
    def test_set_bucket_name(
        self,
        gcs_hook_mock,
        hook_mock,
        tempfile_mock,
        http_mock,
        test_bucket_name,
        dag_maker,
    ):
        http_mock.MediaIoBaseDownload.return_value.next_chunk.return_value = (
            None,
            True,
        )
        tempfile_mock.NamedTemporaryFile.return_value.__enter__.return_value.name = TEMP_FILE_NAME

        with dag_maker(dag_id="test_set_bucket_name", start_date=DEFAULT_DATE) as dag:
            if BUCKET_NAME not in test_bucket_name:

                @dag.task(task_id="taskflow_op")
                def f():
                    return BUCKET_NAME

                taskflow_op = f()

            GoogleCampaignManagerDownloadReportOperator(
                profile_id=PROFILE_ID,
                report_id=REPORT_ID,
                file_id=FILE_ID,
                bucket_name=test_bucket_name if test_bucket_name != "XComArg" else taskflow_op,
                report_name=REPORT_NAME,
                api_version=API_VERSION,
                task_id="test_task",
            )

        dr = dag_maker.create_dagrun()

        for ti in dr.get_task_instances():
            ti.run()

        gcs_hook_mock.return_value.upload.assert_called_once_with(
            bucket_name=BUCKET_NAME,
            object_name=REPORT_NAME + ".gz",
            gzip=True,
            filename=TEMP_FILE_NAME,
            mime_type="text/csv",
        )


class TestGoogleCampaignManagerInsertReportOperator:
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.BaseOperator")
    def test_execute(self, mock_base_op, hook_mock):
        report = {"report": "test"}

        mock_context = {"task_instance": mock.Mock()}

        hook_mock.return_value.insert_report.return_value = {"id": REPORT_ID}

        op = GoogleCampaignManagerInsertReportOperator(
            profile_id=PROFILE_ID,
            report=report,
            api_version=API_VERSION,
            task_id="test_task",
        )
        op.execute(context=mock_context)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=None,
        )
        hook_mock.return_value.insert_report.assert_called_once_with(profile_id=PROFILE_ID, report=report)
        mock_context["task_instance"].xcom_push.assert_called_once_with(key="report_id", value=REPORT_ID)

    def test_prepare_template(self):
        report = {"key": "value"}
        with NamedTemporaryFile("w+", suffix=".json") as f:
            f.write(json.dumps(report))
            f.flush()
            op = GoogleCampaignManagerInsertReportOperator(
                profile_id=PROFILE_ID,
                report=f.name,
                api_version=API_VERSION,
                task_id="test_task",
            )
            op.prepare_template()

        assert isinstance(op.report, dict)
        assert op.report == report


class TestGoogleCampaignManagerRunReportOperator:
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.BaseOperator")
    def test_execute(self, mock_base_op, hook_mock):
        synchronous = True

        mock_context = {"task_instance": mock.Mock()}

        hook_mock.return_value.run_report.return_value = {"id": FILE_ID}

        op = GoogleCampaignManagerRunReportOperator(
            profile_id=PROFILE_ID,
            report_id=REPORT_ID,
            synchronous=synchronous,
            api_version=API_VERSION,
            task_id="test_task",
        )
        op.execute(context=mock_context)
        hook_mock.assert_called_once_with(
            gcp_conn_id=GCP_CONN_ID,
            api_version=API_VERSION,
            impersonation_chain=None,
        )
        hook_mock.return_value.run_report.assert_called_once_with(
            profile_id=PROFILE_ID, report_id=REPORT_ID, synchronous=synchronous
        )
        mock_context["task_instance"].xcom_push.assert_called_once_with(key="file_id", value=FILE_ID)


class TestGoogleCampaignManagerBatchInsertConversionsOperator:
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.BaseOperator")
    def test_execute(self, mock_base_op, hook_mock):
        op = GoogleCampaignManagerBatchInsertConversionsOperator(
            task_id="insert_conversion",
            profile_id=PROFILE_ID,
            conversions=[CONVERSION],
            encryption_source="AD_SERVING",
            encryption_entity_type="DCM_ADVERTISER",
            encryption_entity_id=123456789,
        )
        op.execute(None)
        hook_mock.return_value.conversions_batch_insert.assert_called_once_with(
            profile_id=PROFILE_ID,
            conversions=[CONVERSION],
            encryption_source="AD_SERVING",
            encryption_entity_type="DCM_ADVERTISER",
            encryption_entity_id=123456789,
            max_failed_inserts=0,
        )


class TestGoogleCampaignManagerBatchUpdateConversionOperator:
    @mock.patch(
        "airflow.providers.google.marketing_platform.operators.campaign_manager.GoogleCampaignManagerHook"
    )
    @mock.patch("airflow.providers.google.marketing_platform.operators.campaign_manager.BaseOperator")
    def test_execute(self, mock_base_op, hook_mock):
        op = GoogleCampaignManagerBatchUpdateConversionsOperator(
            task_id="update_conversion",
            profile_id=PROFILE_ID,
            conversions=[CONVERSION],
            encryption_source="AD_SERVING",
            encryption_entity_type="DCM_ADVERTISER",
            encryption_entity_id=123456789,
        )
        op.execute(None)
        hook_mock.return_value.conversions_batch_update.assert_called_once_with(
            profile_id=PROFILE_ID,
            conversions=[CONVERSION],
            encryption_source="AD_SERVING",
            encryption_entity_type="DCM_ADVERTISER",
            encryption_entity_id=123456789,
            max_failed_updates=0,
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
"""
Example DAG using GoogleCloudStorageToGoogleDriveOperator.

Using this operator requires the following additional scopes:
https://www.googleapis.com/auth/drive
"""

from __future__ import annotations

import json
import logging
import os
from datetime import datetime
from pathlib import Path
from typing import Any

from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS

if AIRFLOW_V_3_0_PLUS:
    from airflow.sdk import task
else:
    # Airflow 2 path
    from airflow.decorators import task  # type: ignore[attr-defined,no-redef]
from airflow.models.dag import DAG
from airflow.providers.google.cloud.operators.gcs import GCSCreateBucketOperator, GCSDeleteBucketOperator
from airflow.providers.google.cloud.transfers.gcs_to_gcs import GCSToGCSOperator
from airflow.providers.google.suite.hooks.drive import GoogleDriveHook
from airflow.providers.google.suite.transfers.gcs_to_gdrive import GCSToGoogleDriveOperator

try:
    from airflow.sdk import TriggerRule
except ImportError:
    # Compatibility for Airflow < 3.1
    from airflow.utils.trigger_rule import TriggerRule  # type: ignore[no-redef,attr-defined]

from system.google import DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
from tests_common.test_utils.api_client_helpers import create_airflow_connection, delete_airflow_connection

ENV_ID = os.environ.get("SYSTEM_TESTS_ENV_ID", "default")
PROJECT_ID = os.environ.get("SYSTEM_TESTS_GCP_PROJECT") or DEFAULT_GCP_SYSTEM_TEST_PROJECT_ID
FOLDER_ID = os.environ.get("GCP_GDRIVE_FOLDER_ID", "root")

DAG_ID = "gcs_to_gdrive"

RESOURCES_BUCKET_NAME = "airflow-system-tests-resources"
BUCKET_NAME = f"bucket_{DAG_ID}_{ENV_ID}"
CONNECTION_ID = f"connection_{DAG_ID}_{ENV_ID}"

TMP_PATH = "tmp"
WORK_DIR = f"folder_{DAG_ID}_{ENV_ID}".replace("-", "_")
CURRENT_FOLDER = Path(__file__).parent
LOCAL_PATH = str(Path("gcs"))
FILE_LOCAL_PATH = str(Path(LOCAL_PATH))
FILE_NAME = "example_upload.txt"

log = logging.getLogger(__name__)

with DAG(
    DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
    tags=["example", "gcs", "gdrive"],
) as dag:

    @task
    def create_connection(connection_id: str):
        conn_extra_json = json.dumps(
            {"scope": "https://www.googleapis.com/auth/drive,https://www.googleapis.com/auth/cloud-platform"}
        )
        connection: dict[str, Any] = {"conn_type": "google_cloud_platform", "extra": conn_extra_json}
        create_airflow_connection(
            connection_id=connection_id,
            connection_conf=connection,
        )

    create_connection_task = create_connection(connection_id=CONNECTION_ID)

    create_bucket = GCSCreateBucketOperator(
        task_id="create_bucket", bucket_name=BUCKET_NAME, project_id=PROJECT_ID
    )

    upload_file_1 = GCSToGCSOperator(
        task_id="upload_file_1",
        source_bucket=RESOURCES_BUCKET_NAME,
        source_object=f"{FILE_LOCAL_PATH}/{FILE_NAME}",
        destination_bucket=BUCKET_NAME,
        destination_object=f"{TMP_PATH}/{FILE_NAME}",
        exact_match=True,
    )

    upload_file_2 = GCSToGCSOperator(
        task_id="upload_file_2",
        source_bucket=RESOURCES_BUCKET_NAME,
        source_object=f"{FILE_LOCAL_PATH}/{FILE_NAME}",
        destination_bucket=BUCKET_NAME,
        destination_object=f"{TMP_PATH}/2_{FILE_NAME}",
        exact_match=True,
    )
    # [START howto_operator_gcs_to_gdrive_copy_single_file]
    copy_single_file = GCSToGoogleDriveOperator(
        task_id="copy_single_file",
        gcp_conn_id=CONNECTION_ID,
        source_bucket=BUCKET_NAME,
        source_object=f"{TMP_PATH}/{FILE_NAME}",
        destination_object=f"{WORK_DIR}/copied_{FILE_NAME}",
    )
    # [END howto_operator_gcs_to_gdrive_copy_single_file]

    # [START howto_operator_gcs_to_gdrive_copy_single_file_into_folder]
    copy_single_file_into_folder = GCSToGoogleDriveOperator(
        task_id="copy_single_file_into_folder",
        gcp_conn_id=CONNECTION_ID,
        source_bucket=BUCKET_NAME,
        source_object=f"{TMP_PATH}/{FILE_NAME}",
        destination_object=f"{WORK_DIR}/copied_{FILE_NAME}",
        destination_folder_id=FOLDER_ID,
    )
    # [END howto_operator_gcs_to_gdrive_copy_single_file_into_folder]

    # [START howto_operator_gcs_to_gdrive_copy_files]
    copy_files = GCSToGoogleDriveOperator(
        task_id="copy_files",
        gcp_conn_id=CONNECTION_ID,
        source_bucket=BUCKET_NAME,
        source_object=f"{TMP_PATH}/*",
        destination_object=f"{WORK_DIR}/",
    )
    # [END howto_operator_gcs_to_gdrive_copy_files]

    # [START howto_operator_gcs_to_gdrive_move_files]
    move_files = GCSToGoogleDriveOperator(
        task_id="move_files",
        gcp_conn_id=CONNECTION_ID,
        source_bucket=BUCKET_NAME,
        source_object=f"{TMP_PATH}/*.txt",
        destination_object=f"{WORK_DIR}/",
        move_object=True,
    )
    # [END howto_operator_gcs_to_gdrive_move_files]

    @task(trigger_rule=TriggerRule.ALL_DONE)
    def remove_files_from_drive():
        service = GoogleDriveHook(gcp_conn_id=CONNECTION_ID).get_conn()
        root_path = (
            service.files()
            .list(q=f"name = '{WORK_DIR}' and mimeType = 'application/vnd.google-apps.folder'")
            .execute()
        )
        if files := root_path["files"]:
            batch = service.new_batch_http_request()
            for file in files:
                log.info("Preparing to remove file: {}", file)
                batch.add(service.files().delete(fileId=file["id"]))
            batch.execute()
            log.info("Selected files removed.")

    remove_files_from_drive_task = remove_files_from_drive()

    delete_bucket = GCSDeleteBucketOperator(
        task_id="delete_bucket", bucket_name=BUCKET_NAME, trigger_rule=TriggerRule.ALL_DONE
    )

    @task(task_id="delete_connection")
    def delete_connection(connection_id: str) -> None:
        delete_airflow_connection(connection_id=connection_id)

    delete_connection_task = delete_connection(connection_id=CONNECTION_ID)

    # TEST SETUP
    create_bucket >> [upload_file_1, upload_file_2]
    (
        [upload_file_1, upload_file_2, create_connection_task]
        # TEST BODY
        >> copy_single_file
        >> copy_single_file_into_folder
        >> copy_files
        >> move_files
        # TEST TEARDOWN
        >> remove_files_from_drive_task
        >> [delete_bucket, delete_connection_task]
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

import pytest

# For no Pydantic environment, we need to skip the tests
pytest.importorskip("google.cloud.aiplatform_v1")

from google.api_core.gapic_v1.method import DEFAULT

from airflow.providers.google.cloud.hooks.vertex_ai.dataset import DatasetHook

from unit.google.cloud.utils.base_gcp_mock import (
    mock_base_gcp_hook_default_project_id,
    mock_base_gcp_hook_no_default_project_id,
)

TEST_GCP_CONN_ID: str = "test-gcp-conn-id"
TEST_REGION: str = "test-region"
TEST_PROJECT_ID: str = "test-project-id"
TEST_PIPELINE_JOB: dict = {}
TEST_PIPELINE_JOB_ID: str = "test-pipeline-job-id"
TEST_TRAINING_PIPELINE: dict = {}
TEST_TRAINING_PIPELINE_NAME: str = "test-training-pipeline"
TEST_DATASET: dict = {}
TEST_DATASET_ID: str = "test-dataset-id"
TEST_EXPORT_CONFIG: dict = {}
TEST_ANNOTATION_SPEC: str = "test-annotation-spec"
TEST_IMPORT_CONFIGS: dict = {}
TEST_DATA_ITEM: str = "test-data-item"
TEST_UPDATE_MASK: dict = {}

BASE_STRING = "airflow.providers.google.common.hooks.base_google.{}"
DATASET_STRING = "airflow.providers.google.cloud.hooks.vertex_ai.dataset.{}"


class TestVertexAIWithDefaultProjectIdHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"), new=mock_base_gcp_hook_default_project_id
        ):
            self.hook = DatasetHook(gcp_conn_id=TEST_GCP_CONN_ID)

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_create_dataset(self, mock_client) -> None:
        self.hook.create_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.create_dataset.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                dataset=TEST_DATASET,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_REGION)

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_delete_dataset(self, mock_client) -> None:
        self.hook.delete_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.delete_dataset.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_export_data(self, mock_client) -> None:
        self.hook.export_data(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            export_config=TEST_EXPORT_CONFIG,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.export_data.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
                export_config=TEST_EXPORT_CONFIG,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_get_annotation_spec(self, mock_client) -> None:
        self.hook.get_annotation_spec(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            annotation_spec=TEST_ANNOTATION_SPEC,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.get_annotation_spec.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.annotation_spec_path.return_value,
                read_mask=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.annotation_spec_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID, TEST_ANNOTATION_SPEC
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_get_dataset(self, mock_client) -> None:
        self.hook.get_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.get_dataset.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
                read_mask=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_import_data(self, mock_client) -> None:
        self.hook.import_data(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            import_configs=TEST_IMPORT_CONFIGS,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.import_data.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
                import_configs=TEST_IMPORT_CONFIGS,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_list_annotations(self, mock_client) -> None:
        self.hook.list_annotations(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            data_item=TEST_DATA_ITEM,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_annotations.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.data_item_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.data_item_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID, TEST_DATA_ITEM
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_list_data_items(self, mock_client) -> None:
        self.hook.list_data_items(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_data_items.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.dataset_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_list_datasets(self, mock_client) -> None:
        self.hook.list_datasets(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_datasets.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_REGION)

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_update_dataset(self, mock_client) -> None:
        self.hook.update_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset_id=TEST_DATASET_ID,
            dataset=TEST_DATASET,
            update_mask=TEST_UPDATE_MASK,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.update_dataset.assert_called_once_with(
            request=dict(
                dataset=TEST_DATASET,
                update_mask=TEST_UPDATE_MASK,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )


class TestVertexAIWithoutDefaultProjectIdHook:
    def setup_method(self):
        with mock.patch(
            BASE_STRING.format("GoogleBaseHook.__init__"), new=mock_base_gcp_hook_no_default_project_id
        ):
            self.hook = DatasetHook(gcp_conn_id=TEST_GCP_CONN_ID)

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_create_dataset(self, mock_client) -> None:
        self.hook.create_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.create_dataset.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                dataset=TEST_DATASET,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_REGION)

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_delete_dataset(self, mock_client) -> None:
        self.hook.delete_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.delete_dataset.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_export_data(self, mock_client) -> None:
        self.hook.export_data(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            export_config=TEST_EXPORT_CONFIG,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.export_data.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
                export_config=TEST_EXPORT_CONFIG,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_get_annotation_spec(self, mock_client) -> None:
        self.hook.get_annotation_spec(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            annotation_spec=TEST_ANNOTATION_SPEC,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.get_annotation_spec.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.annotation_spec_path.return_value,
                read_mask=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.annotation_spec_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID, TEST_ANNOTATION_SPEC
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_get_dataset(self, mock_client) -> None:
        self.hook.get_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.get_dataset.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
                read_mask=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_import_data(self, mock_client) -> None:
        self.hook.import_data(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            import_configs=TEST_IMPORT_CONFIGS,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.import_data.assert_called_once_with(
            request=dict(
                name=mock_client.return_value.dataset_path.return_value,
                import_configs=TEST_IMPORT_CONFIGS,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_list_annotations(self, mock_client) -> None:
        self.hook.list_annotations(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
            data_item=TEST_DATA_ITEM,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_annotations.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.data_item_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.data_item_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID, TEST_DATA_ITEM
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_list_data_items(self, mock_client) -> None:
        self.hook.list_data_items(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset=TEST_DATASET_ID,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_data_items.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.dataset_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_list_datasets(self, mock_client) -> None:
        self.hook.list_datasets(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.list_datasets.assert_called_once_with(
            request=dict(
                parent=mock_client.return_value.common_location_path.return_value,
                filter=None,
                page_size=None,
                page_token=None,
                read_mask=None,
                order_by=None,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.common_location_path.assert_called_once_with(TEST_PROJECT_ID, TEST_REGION)

    @mock.patch(DATASET_STRING.format("DatasetHook.get_dataset_service_client"))
    def test_update_dataset(self, mock_client) -> None:
        self.hook.update_dataset(
            project_id=TEST_PROJECT_ID,
            region=TEST_REGION,
            dataset_id=TEST_DATASET_ID,
            dataset=TEST_DATASET,
            update_mask=TEST_UPDATE_MASK,
        )
        mock_client.assert_called_once_with(TEST_REGION)
        mock_client.return_value.update_dataset.assert_called_once_with(
            request=dict(
                dataset=TEST_DATASET,
                update_mask=TEST_UPDATE_MASK,
            ),
            metadata=(),
            retry=DEFAULT,
            timeout=None,
        )
        mock_client.return_value.dataset_path.assert_called_once_with(
            TEST_PROJECT_ID, TEST_REGION, TEST_DATASET_ID
        )



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import importlib
from unittest import mock

import pytest

# Do not run the tests when FAB / Flask is not installed
pytest.importorskip("flask_session")

from google.auth.exceptions import GoogleAuthError

from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS

if not AIRFLOW_V_3_0_PLUS:
    pytest.skip(
        "``providers/google/tests/unit/google/common/auth_backend/test_google_openid.py`` is only compatible with Airflow 3.X.",
        allow_module_level=True,
    )

from tests_common.test_utils.config import conf_vars


@pytest.fixture(scope="module")
def google_openid_app():
    if importlib.util.find_spec("flask_session") is None:
        return None

    def factory():
        with conf_vars(
            {
                ("fab", "auth_backends"): "airflow.providers.google.common.auth_backend.google_openid",
                (
                    "core",
                    "auth_manager",
                ): "airflow.providers.fab.auth_manager.fab_auth_manager.FabAuthManager",
            }
        ):
            from airflow.providers.fab.www.app import create_app

            _app = create_app(enable_plugins=False)
            _app.config["AUTH_ROLE_PUBLIC"] = None
            return _app

    return factory()


def delete_user(app, username):
    appbuilder = app.appbuilder
    for user in appbuilder.sm.get_all_users():
        if user.username == username:
            appbuilder.sm.del_register_user(user)
            break


@pytest.fixture(scope="module")
def admin_user(google_openid_app):
    if importlib.util.find_spec("airflow.providers.fab") is None:
        return
    appbuilder = google_openid_app.appbuilder
    role_admin = appbuilder.sm.find_role("Admin")
    delete_user(google_openid_app, "test")
    appbuilder.sm.add_user(
        username="test",
        first_name="test",
        last_name="test",
        email="test@fab.org",
        role=role_admin,
        password="test",
    )
    return role_admin


@pytest.mark.skipif(
    importlib.util.find_spec("airflow.providers.fab") is None, reason="FAB provider is not installed"
)
@pytest.mark.db_test
class TestGoogleOpenID:
    @pytest.fixture(autouse=True)
    def _set_attrs(self, google_openid_app, admin_user) -> None:
        self.app = google_openid_app
        self.admin_user = admin_user

    @mock.patch("google.oauth2.id_token.verify_token")
    def test_success(self, mock_verify_token):
        mock_verify_token.return_value = {
            "iss": "accounts.google.com",
            "email_verified": True,
            "email": "test@fab.org",
        }

        with self.app.test_client() as test_client:
            response = test_client.get("/fab/v1/users", headers={"Authorization": "bearer JWT_TOKEN"})

            assert response.status_code == 200

    @pytest.mark.parametrize("auth_header", ["bearer", "JWT_TOKEN", "bearer "])
    @mock.patch("google.oauth2.id_token.verify_token")
    def test_malformed_headers(self, mock_verify_token, auth_header):
        mock_verify_token.return_value = {
            "iss": "accounts.google.com",
            "email_verified": True,
            "email": "test@fab.org",
        }

        with self.app.test_client() as test_client:
            response = test_client.get("/fab/v1/users", headers={"Authorization": auth_header})

        assert response.status_code == 401

    @mock.patch("google.oauth2.id_token.verify_token")
    def test_invalid_iss_in_jwt_token(self, mock_verify_token):
        mock_verify_token.return_value = {
            "iss": "INVALID",
            "email_verified": True,
            "email": "test@fab.org",
        }

        with self.app.test_client() as test_client:
            response = test_client.get("/fab/v1/users", headers={"Authorization": "bearer JWT_TOKEN"})

        assert response.status_code == 401

    @mock.patch("google.oauth2.id_token.verify_token")
    def test_user_not_exists(self, mock_verify_token):
        mock_verify_token.return_value = {
            "iss": "accounts.google.com",
            "email_verified": True,
            "email": "invalid@fab.org",
        }

        with self.app.test_client() as test_client:
            response = test_client.get("/fab/v1/users", headers={"Authorization": "bearer JWT_TOKEN"})

        assert response.status_code == 401

    @conf_vars({("fab", "auth_backends"): "airflow.providers.google.common.auth_backend.google_openid"})
    def test_missing_id_token(self):
        with self.app.test_client() as test_client:
            response = test_client.get("/fab/v1/users")

        assert response.status_code == 401

    @conf_vars({("fab", "auth_backends"): "airflow.providers.google.common.auth_backend.google_openid"})
    @mock.patch("google.oauth2.id_token.verify_token")
    def test_invalid_id_token(self, mock_verify_token):
        mock_verify_token.side_effect = GoogleAuthError("Invalid token")

        with self.app.test_client() as test_client:
            response = test_client.get("/fab/v1/users", headers={"Authorization": "bearer JWT_TOKEN"})

        assert response.status_code == 401



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from unittest import mock

from airflow.providers.google.marketing_platform.hooks.analytics_admin import GoogleAnalyticsAdminHook

from unit.google.cloud.utils.base_gcp_mock import mock_base_gcp_hook_default_project_id

GCP_CONN_ID = "test_gcp_conn_id"
IMPERSONATION_CHAIN = ["ACCOUNT_1", "ACCOUNT_2", "ACCOUNT_3"]
TEST_PROPERTY_ID = "123456789"
TEST_PROPERTY_NAME = f"properties/{TEST_PROPERTY_ID}"
TEST_DATASTREAM_ID = "987654321"
TEST_DATASTREAM_NAME = f"properties/{TEST_PROPERTY_ID}/dataStreams/{TEST_DATASTREAM_ID}"
ANALYTICS_HOOK_PATH = "airflow.providers.google.marketing_platform.hooks.analytics_admin"


class TestGoogleAnalyticsAdminHook:
    def setup_method(self):
        with mock.patch(
            "airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__",
            new=mock_base_gcp_hook_default_project_id,
        ):
            self.hook = GoogleAnalyticsAdminHook(GCP_CONN_ID)

    @mock.patch("airflow.providers.google.common.hooks.base_google.GoogleBaseHook.__init__")
    def test_init(self, mock_base_init):
        GoogleAnalyticsAdminHook(
            GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )
        mock_base_init.assert_called_once_with(
            GCP_CONN_ID,
            impersonation_chain=IMPERSONATION_CHAIN,
        )

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.CLIENT_INFO")
    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_credentials")
    @mock.patch(f"{ANALYTICS_HOOK_PATH}.AnalyticsAdminServiceClient")
    def test_get_conn(self, mock_client, get_credentials, mock_client_info):
        mock_credentials = mock.MagicMock()
        get_credentials.return_value = mock_credentials

        result = self.hook.get_conn()

        mock_client.assert_called_once_with(credentials=mock_credentials, client_info=mock_client_info)
        assert self.hook._conn == result

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_conn")
    def test_list_accounts(self, mock_get_conn):
        list_accounts_expected = mock.MagicMock()
        mock_list_accounts = mock_get_conn.return_value.list_accounts
        mock_list_accounts.return_value = list_accounts_expected
        mock_page_size, mock_page_token, mock_show_deleted, mock_retry, mock_timeout, mock_metadata = (
            mock.MagicMock() for _ in range(6)
        )

        request = {
            "page_size": mock_page_size,
            "page_token": mock_page_token,
            "show_deleted": mock_show_deleted,
        }

        list_accounts_received = self.hook.list_accounts(
            page_size=mock_page_size,
            page_token=mock_page_token,
            show_deleted=mock_show_deleted,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        mock_list_accounts.assert_called_once_with(
            request=request, retry=mock_retry, timeout=mock_timeout, metadata=mock_metadata
        )
        assert list_accounts_received == list_accounts_expected

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_conn")
    def test_create_property(self, mock_get_conn):
        property_expected = mock.MagicMock()

        mock_create_property = mock_get_conn.return_value.create_property
        mock_create_property.return_value = property_expected
        mock_property, mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(4))

        property_created = self.hook.create_property(
            analytics_property=mock_property, retry=mock_retry, timeout=mock_timeout, metadata=mock_metadata
        )

        request = {"property": mock_property}
        mock_create_property.assert_called_once_with(
            request=request, retry=mock_retry, timeout=mock_timeout, metadata=mock_metadata
        )
        assert property_created == property_expected

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_conn")
    def test_delete_property(self, mock_get_conn):
        property_expected = mock.MagicMock()
        mock_delete_property = mock_get_conn.return_value.delete_property
        mock_delete_property.return_value = property_expected
        mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(3))

        property_deleted = self.hook.delete_property(
            property_id=TEST_PROPERTY_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        request = {"name": TEST_PROPERTY_NAME}
        mock_delete_property.assert_called_once_with(
            request=request,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )
        assert property_deleted == property_expected

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_conn")
    def test_create_data_stream(self, mock_get_conn):
        data_stream_expected = mock.MagicMock()
        mock_create_data_stream = mock_get_conn.return_value.create_data_stream
        mock_create_data_stream.return_value = data_stream_expected
        mock_data_stream, mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(4))

        data_stream_created = self.hook.create_data_stream(
            property_id=TEST_PROPERTY_ID,
            data_stream=mock_data_stream,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )

        request = {"parent": TEST_PROPERTY_NAME, "data_stream": mock_data_stream}
        mock_create_data_stream.assert_called_once_with(
            request=request, retry=mock_retry, timeout=mock_timeout, metadata=mock_metadata
        )
        assert data_stream_created == data_stream_expected

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_conn")
    def test_delete_data_stream(self, mock_get_conn):
        mock_retry, mock_timeout, mock_metadata = (mock.MagicMock() for _ in range(3))

        self.hook.delete_data_stream(
            property_id=TEST_PROPERTY_ID,
            data_stream_id=TEST_DATASTREAM_ID,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )

        request = {"name": TEST_DATASTREAM_NAME}
        mock_get_conn.return_value.delete_data_stream.assert_called_once_with(
            request=request,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )

    @mock.patch(f"{ANALYTICS_HOOK_PATH}.GoogleAnalyticsAdminHook.get_conn")
    def test_list_ads_links(self, mock_get_conn):
        mock_page_size, mock_page_token, mock_retry, mock_timeout, mock_metadata = (
            mock.MagicMock() for _ in range(5)
        )

        self.hook.list_google_ads_links(
            property_id=TEST_PROPERTY_ID,
            page_size=mock_page_size,
            page_token=mock_page_token,
            retry=mock_retry,
            timeout=mock_timeout,
            metadata=mock_metadata,
        )

        request = {"parent": TEST_PROPERTY_NAME, "page_size": mock_page_size, "page_token": mock_page_token}
        mock_get_conn.return_value.list_google_ads_links.assert_called_once_with(
            request=request, retry=mock_retry, timeout=mock_timeout, metadata=mock_metadata
        )



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from typing import Any
from unittest import mock

import pytest
from google.api_core.gapic_v1.method import DEFAULT
from google.cloud.pubsub_v1.types import ReceivedMessage

from airflow.exceptions import TaskDeferred
from airflow.providers.google.cloud.operators.pubsub import (
    PubSubCreateSubscriptionOperator,
    PubSubCreateTopicOperator,
    PubSubDeleteSubscriptionOperator,
    PubSubDeleteTopicOperator,
    PubSubPublishMessageOperator,
    PubSubPullOperator,
)

TASK_ID = "test-task-id"
TEST_PROJECT = "test-project"
TEST_TOPIC = "test-topic"
TEST_SUBSCRIPTION = "test-subscription"
TEST_MESSAGES = [
    {"data": b"Hello, World!", "attributes": {"type": "greeting"}},
    {"data": b"Knock, knock"},
    {"attributes": {"foo": ""}},
]
TEST_MESSAGES_ORDERING_KEY = [
    {"data": b"Hello, World!", "attributes": {"ordering_key": "key"}},
]


class TestPubSubTopicCreateOperator:
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_failifexists(self, mock_hook):
        operator = PubSubCreateTopicOperator(
            task_id=TASK_ID, project_id=TEST_PROJECT, topic=TEST_TOPIC, fail_if_exists=True
        )

        context = mock.MagicMock()
        operator.execute(context=context)
        mock_hook.return_value.create_topic.assert_called_once_with(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            fail_if_exists=True,
            labels=None,
            message_storage_policy=None,
            kms_key_name=None,
            schema_settings=None,
            message_retention_duration=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_succeedifexists(self, mock_hook):
        operator = PubSubCreateTopicOperator(
            task_id=TASK_ID, project_id=TEST_PROJECT, topic=TEST_TOPIC, fail_if_exists=False
        )

        context = mock.MagicMock()
        operator.execute(context=context)
        mock_hook.return_value.create_topic.assert_called_once_with(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            fail_if_exists=False,
            labels=None,
            message_storage_policy=None,
            kms_key_name=None,
            schema_settings=None,
            message_retention_duration=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestPubSubTopicDeleteOperator:
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute(self, mock_hook):
        operator = PubSubDeleteTopicOperator(task_id=TASK_ID, project_id=TEST_PROJECT, topic=TEST_TOPIC)

        operator.execute(None)
        mock_hook.return_value.delete_topic.assert_called_once_with(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            fail_if_not_exists=False,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestPubSubSubscriptionCreateOperator:
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute(self, mock_hook):
        operator = PubSubCreateSubscriptionOperator(
            task_id=TASK_ID, project_id=TEST_PROJECT, topic=TEST_TOPIC, subscription=TEST_SUBSCRIPTION
        )
        mock_hook.return_value.create_subscription.return_value = TEST_SUBSCRIPTION
        context = mock.MagicMock()
        response = operator.execute(context=context)
        mock_hook.return_value.create_subscription.assert_called_once_with(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            subscription=TEST_SUBSCRIPTION,
            subscription_project_id=None,
            ack_deadline_secs=10,
            fail_if_exists=False,
            push_config=None,
            retain_acked_messages=None,
            message_retention_duration=None,
            labels=None,
            enable_message_ordering=False,
            expiration_policy=None,
            filter_=None,
            dead_letter_policy=None,
            retry_policy=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )
        assert response == TEST_SUBSCRIPTION

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute_different_project_ids(self, mock_hook):
        another_project = "another-project"
        operator = PubSubCreateSubscriptionOperator(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            subscription=TEST_SUBSCRIPTION,
            subscription_project_id=another_project,
            task_id=TASK_ID,
        )
        mock_hook.return_value.create_subscription.return_value = TEST_SUBSCRIPTION
        context = mock.MagicMock()
        response = operator.execute(context=context)
        mock_hook.return_value.create_subscription.assert_called_once_with(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            subscription=TEST_SUBSCRIPTION,
            subscription_project_id=another_project,
            ack_deadline_secs=10,
            fail_if_exists=False,
            push_config=None,
            retain_acked_messages=None,
            message_retention_duration=None,
            labels=None,
            enable_message_ordering=False,
            expiration_policy=None,
            filter_=None,
            dead_letter_policy=None,
            retry_policy=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )
        assert response == TEST_SUBSCRIPTION

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute_no_subscription(self, mock_hook):
        operator = PubSubCreateSubscriptionOperator(
            task_id=TASK_ID, project_id=TEST_PROJECT, topic=TEST_TOPIC
        )
        mock_hook.return_value.create_subscription.return_value = TEST_SUBSCRIPTION
        context = mock.MagicMock()
        response = operator.execute(context=context)
        mock_hook.return_value.create_subscription.assert_called_once_with(
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            subscription=None,
            subscription_project_id=None,
            ack_deadline_secs=10,
            fail_if_exists=False,
            push_config=None,
            retain_acked_messages=None,
            message_retention_duration=None,
            labels=None,
            enable_message_ordering=False,
            expiration_policy=None,
            filter_=None,
            dead_letter_policy=None,
            retry_policy=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )
        assert response == TEST_SUBSCRIPTION

    @pytest.mark.parametrize(
        "project_id, subscription, subscription_project_id, expected_input, expected_output",
        [
            (
                TEST_PROJECT,
                TEST_SUBSCRIPTION,
                None,
                f"topic:{TEST_PROJECT}:{TEST_TOPIC}",
                f"subscription:{TEST_PROJECT}:{TEST_SUBSCRIPTION}",
            ),
            (
                TEST_PROJECT,
                TEST_SUBSCRIPTION,
                "another-project",
                f"topic:{TEST_PROJECT}:{TEST_TOPIC}",
                f"subscription:another-project:{TEST_SUBSCRIPTION}",
            ),
            (
                TEST_PROJECT,
                None,
                None,
                f"topic:{TEST_PROJECT}:{TEST_TOPIC}",
                f"subscription:{TEST_PROJECT}:generated",
            ),
            (
                TEST_PROJECT,
                None,
                "another-project",
                f"topic:{TEST_PROJECT}:{TEST_TOPIC}",
                "subscription:another-project:generated",
            ),
            (
                None,
                None,
                None,
                f"topic:connection-project:{TEST_TOPIC}",
                "subscription:connection-project:generated",
            ),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_get_openlineage_facets(
        self,
        mock_hook,
        project_id,
        subscription,
        subscription_project_id,
        expected_input,
        expected_output,
    ):
        operator = PubSubCreateSubscriptionOperator(
            task_id=TASK_ID,
            project_id=project_id,
            topic=TEST_TOPIC,
            subscription=subscription,
            subscription_project_id=subscription_project_id,
        )
        mock_hook.return_value.create_subscription.return_value = subscription or "generated"
        mock_hook.return_value.project_id = project_id or "connection-project"
        context = mock.MagicMock()
        response = operator.execute(context=context)
        mock_hook.return_value.create_subscription.assert_called_once_with(
            project_id=project_id,
            topic=TEST_TOPIC,
            subscription=subscription,
            subscription_project_id=subscription_project_id,
            ack_deadline_secs=10,
            fail_if_exists=False,
            push_config=None,
            retain_acked_messages=None,
            message_retention_duration=None,
            labels=None,
            enable_message_ordering=False,
            expiration_policy=None,
            filter_=None,
            dead_letter_policy=None,
            retry_policy=None,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )

        if subscription:
            assert response == TEST_SUBSCRIPTION
        else:
            assert response == "generated"

        result = operator.get_openlineage_facets_on_complete(operator)
        assert not result.run_facets
        assert not result.job_facets
        assert len(result.inputs) == 1
        assert result.inputs[0].namespace == "pubsub"
        assert result.inputs[0].name == expected_input
        assert len(result.outputs) == 1
        assert result.outputs[0].namespace == "pubsub"
        assert result.outputs[0].name == expected_output


class TestPubSubSubscriptionDeleteOperator:
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute(self, mock_hook):
        operator = PubSubDeleteSubscriptionOperator(
            task_id=TASK_ID, project_id=TEST_PROJECT, subscription=TEST_SUBSCRIPTION
        )

        operator.execute(None)
        mock_hook.return_value.delete_subscription.assert_called_once_with(
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
            fail_if_not_exists=False,
            retry=DEFAULT,
            timeout=None,
            metadata=(),
        )


class TestPubSubPublishOperator:
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_publish(self, mock_hook):
        operator = PubSubPublishMessageOperator(
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            messages=TEST_MESSAGES,
        )

        operator.execute(None)
        mock_hook.return_value.publish.assert_called_once_with(
            project_id=TEST_PROJECT, topic=TEST_TOPIC, messages=TEST_MESSAGES
        )

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_publish_with_ordering_key(self, mock_hook):
        operator = PubSubPublishMessageOperator(
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            topic=TEST_TOPIC,
            messages=TEST_MESSAGES_ORDERING_KEY,
            enable_message_ordering=True,
        )

        operator.execute(None)
        mock_hook.return_value.publish.assert_called_once_with(
            project_id=TEST_PROJECT, topic=TEST_TOPIC, messages=TEST_MESSAGES_ORDERING_KEY
        )

    @pytest.mark.parametrize(
        "project_id, expected_dataset",
        [
            # 1. project_id provided
            (TEST_PROJECT, f"topic:{TEST_PROJECT}:{TEST_TOPIC}"),
            # 2. project_id not provided (use project_id from connection)
            (None, f"topic:connection-project:{TEST_TOPIC}"),
        ],
    )
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_get_openlineage_facets(self, mock_hook, project_id, expected_dataset):
        operator = PubSubPublishMessageOperator(
            task_id=TASK_ID,
            project_id=project_id,
            topic=TEST_TOPIC,
            messages=TEST_MESSAGES,
        )

        operator.execute(None)
        mock_hook.return_value.publish.assert_called_once_with(
            project_id=project_id, topic=TEST_TOPIC, messages=TEST_MESSAGES
        )
        mock_hook.return_value.project_id = project_id or "connection-project"

        result = operator.get_openlineage_facets_on_complete(operator)
        assert not result.run_facets
        assert not result.job_facets
        assert len(result.inputs) == 0
        assert len(result.outputs) == 1
        assert result.outputs[0].namespace == "pubsub"
        assert result.outputs[0].name == expected_dataset


class TestPubSubPullOperator:
    def _generate_messages(self, count):
        return [
            ReceivedMessage(
                ack_id=f"{i}",
                message={
                    "data": f"Message {i}".encode(),
                    "attributes": {"type": "generated message"},
                },
            )
            for i in range(1, count + 1)
        ]

    def _generate_dicts(self, count):
        return [ReceivedMessage.to_dict(m) for m in self._generate_messages(count)]

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute_no_messages(self, mock_hook):
        operator = PubSubPullOperator(
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
        )

        mock_hook.return_value.pull.return_value = []
        assert operator.execute({}) == []

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute_with_ack_messages(self, mock_hook):
        operator = PubSubPullOperator(
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
            ack_messages=True,
        )

        generated_messages = self._generate_messages(5)
        generated_dicts = self._generate_dicts(5)
        mock_hook.return_value.pull.return_value = generated_messages

        assert generated_dicts == operator.execute({})
        mock_hook.return_value.acknowledge.assert_called_once_with(
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
            messages=generated_messages,
        )

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute_with_messages_callback(self, mock_hook):
        generated_messages = self._generate_messages(5)
        messages_callback_return_value = "asdfg"

        def messages_callback(
            pulled_messages: list[ReceivedMessage],
            context: dict[str, Any],
        ):
            assert pulled_messages == generated_messages

            assert isinstance(context, dict)
            for key in context.keys():
                assert isinstance(key, str)

            return messages_callback_return_value

        messages_callback = mock.Mock(side_effect=messages_callback)

        operator = PubSubPullOperator(
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
            messages_callback=messages_callback,
        )

        mock_hook.return_value.pull.return_value = generated_messages

        response = operator.execute({})
        mock_hook.return_value.pull.assert_called_once_with(
            project_id=TEST_PROJECT, subscription=TEST_SUBSCRIPTION, max_messages=5, return_immediately=True
        )

        messages_callback.assert_called_once()

        assert response == messages_callback_return_value

    @pytest.mark.db_test
    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_execute_deferred(self, mock_hook, create_task_instance_of_operator):
        """
        Asserts that a task is deferred and a PubSubPullOperator will be fired
        when the PubSubPullOperator is executed with deferrable=True.
        """
        ti = create_task_instance_of_operator(
            PubSubPullOperator,
            dag_id="dag_id",
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
            deferrable=True,
        )
        with pytest.raises(TaskDeferred) as _:
            ti.task.execute(mock.MagicMock())

    @mock.patch("airflow.providers.google.cloud.operators.pubsub.PubSubHook")
    def test_get_openlineage_facets(self, mock_hook):
        operator = PubSubPullOperator(
            task_id=TASK_ID,
            project_id=TEST_PROJECT,
            subscription=TEST_SUBSCRIPTION,
        )

        generated_messages = self._generate_messages(5)
        generated_dicts = self._generate_dicts(5)
        mock_hook.return_value.pull.return_value = generated_messages

        assert generated_dicts == operator.execute({})
        mock_hook.return_value.pull.assert_called_once_with(
            project_id=TEST_PROJECT, subscription=TEST_SUBSCRIPTION, max_messages=5, return_immediately=True
        )

        result = operator.get_openlineage_facets_on_complete(operator)
        assert not result.run_facets
        assert not result.job_facets
        assert len(result.inputs) == 0
        assert len(result.outputs) == 1
        assert result.outputs[0].namespace == "pubsub"
        assert result.outputs[0].name == f"subscription:{TEST_PROJECT}:{TEST_SUBSCRIPTION}"
